==> fred_regal-1.8.0.tar.gz: fred_regal-1.8.0/regal/nsset.py <==
"""Nsset client for registry service."""

import warnings
from datetime import datetime
from ipaddress import IPv4Address, IPv6Address, ip_address
from typing import Any, AsyncIterable, Dict, List, Optional, Sequence, Union, cast

import grpc
from fred_api.registry.nsset.nsset_history_types_pb2 import NssetHistoryReply, NssetHistoryRequest
from fred_api.registry.nsset.nsset_info_types_pb2 import (
    CheckDnsHostRequest,
    DnsHost as DnsHostMessage,
    NssetIdReply,
    NssetIdRequest,
    NssetInfoReply,
    NssetInfoRequest,
)
from fred_api.registry.nsset.nsset_state_history_types_pb2 import NssetStateHistoryReply, NssetStateHistoryRequest
from fred_api.registry.nsset.nsset_state_types_pb2 import NssetStateReply, NssetStateRequest
from fred_api.registry.nsset.service_nsset_grpc_pb2_grpc import NssetStub
from fred_types import BaseModel, ContactRef, NssetHistoryId, NssetId, NssetRef
from google.protobuf.empty_pb2 import Empty

from .common import ObjectEvents, RegistryDecoder
from .exceptions import InvalidHistoryInterval, NssetDoesNotExist
from .object import ObjectStateFlag, RegistryObjectClient
from .utils import _idna_encode


class DnsHost(BaseModel):
    """Dns host info."""

    fqdn: str
    ip_addresses: List[Union[IPv4Address, IPv6Address]]


class Nsset(BaseModel):
    """Nsset info."""

    nsset_id: NssetId
    nsset_handle: str
    nsset_history_id: Optional[NssetHistoryId] = None
    sponsoring_registrar: Optional[str] = None
    dns_hosts: Sequence[DnsHost] = []
    technical_contacts: Sequence[ContactRef] = []
    technical_check_level: Optional[int] = None
    auth_info: Optional[str] = None
    events: Optional[ObjectEvents] = None


class NssetDecoder(RegistryDecoder):
    """Nsset service decoder."""

    def __init__(self) -> None:
        super().__init__()
        self.set_decoder(DnsHostMessage, self._decode_dns_host)
        self.set_exception_decoder(NssetIdReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetInfoReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetHistoryReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetHistoryReply.Exception.InvalidHistoryInterval, InvalidHistoryInterval)
        self.set_exception_decoder(NssetStateReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetStateHistoryReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetStateHistoryReply.Exception.InvalidHistoryInterval, InvalidHistoryInterval)

    def _decode_dns_host(self, value: DnsHostMessage) -> DnsHost:
        """Decode DNS host."""
        return DnsHost(
            fqdn=self.decode(value.fqdn),
            ip_addresses=[ip_address(address) for address in self.decode(value.ip_addresses)],
        )


class NssetClient(RegistryObjectClient[NssetId, NssetHistoryId]):
    """Nsset service gRPC client."""

    stub_cls = NssetStub
    decoder_cls = NssetDecoder
    service = "Nsset"
    object_id = "nsset_id"
    object_history_id = "nsset_history_id"
    object_handle = "nsset_handle"

    def __init__(self, netloc: str, credentials: Optional[grpc.ChannelCredentials] = None, **kwargs: Any) -> None:
        """Initialize nsset client instance.

        Args:
            netloc: Netloc of nsset service server.
            credentials: Optional credentials.
            kwargs: Other client arguments.
        """
        super().__init__(netloc, credentials=credentials, **kwargs)

    async def list(self) -> AsyncIterable[NssetRef]:
        """Iterate through all nssets in domain registry.

        Returns:
            Async iterable over domain refs.
        """
        # XXX: Bug in python 3.9 coverage
        async for refs in self.call_stream("list_nssets", Empty()):  # pragma: no cover
            for ref in refs:
                yield ref

    async def get_nsset_id(self, handle: str) -> NssetId:
        """Get nsset id.

        Args:
            handle: Nsset handle.

        Raises:
            NssetDoesNotExist: If nsset does not exist.
        """
        return NssetId(await self.get_object_id(request=NssetIdRequest(), method="get_nsset_id", handle=handle))

    async def get_nsset_info(self, nsset_id: NssetId, history_id: Optional[NssetHistoryId] = None) -> Nsset:
        """Get nsset info.

        Args:
            nsset_id: Nsset ID.
            history_id: Nsset history ID.

        Raises:
            NssetDoesNotExist: If nsset does not exist.
        """
        result = await self.get_object_info(
            request=NssetInfoRequest(),
            method="get_nsset_info",
            object_id=nsset_id,
            history_id=history_id,
        )
        result.pop("technical_contacts_old", None)
        return Nsset(**result)

    async def get_nsset_state(self, nsset_id: NssetId, *, internal: Optional[bool] = None) -> Dict[str, bool]:
        """Get nsset state consisting of state flags and their boolean values.

        Args:
            nsset_id: Nsset ID.
            internal: Whether to include internal states in the result.

        Raises:
            NssetDoesNotExist: If nsset does not exist.
        """
        if internal is None:  # pragma: no cover
            warnings.warn(
                "Default value of internal flag will change to False in regal 2.0.", DeprecationWarning, stacklevel=2
            )
            internal = True

        return await self.get_object_state(
            request=NssetStateRequest(), method="get_nsset_state", object_id=nsset_id, internal=internal
        )

    async def get_nsset_history(
        self,
        nsset_id: NssetId,
        start: Optional[Union[NssetHistoryId, datetime]] = None,
        end: Optional[Union[NssetHistoryId, datetime]] = None,
    ) -> Dict[str, Any]:
        """Get nsset history timeline.

        Args:
            nsset_id: Nsset ID.
            start: Datetime or history_id of history interval start.
            end: Datetime or history_id of history interval end.

        Raises:
            NssetDoesNotExist: If nsset does not exist.
            InvalidHistoryInterval: In case of invalid history interval.
        """
        return await self.get_object_history(
            request=NssetHistoryRequest(),
            method="get_nsset_history",
            object_id=nsset_id,
            start=start,
            end=end,
        )

    async def get_nsset_state_history(
        self,
        nsset_id: NssetId,
        start: Optional[Union[NssetHistoryId, datetime]] = None,
        end: Optional[Union[NssetHistoryId, datetime]] = None,
    ) -> Dict[str, Any]:
        """Get nsset state history timeline.

        Args:
            nsset_id: Nsset ID.
            start: Datetime or history_id of history interval start.
            end: Datetime or history_id of history interval end.

        Raises:
            NssetDoesNotExist: If nsset does not exist.
            InvalidHistoryInterval: In case of invalid history interval.
        """
        return await self.get_object_state_history(
            request=NssetStateHistoryRequest(),
            method="get_nsset_state_history",
            object_id=nsset_id,
            start=start,
            end=end,
        )

    async def get_state_flags(self) -> Dict[str, ObjectStateFlag]:
        """Get object state flags and their properties."""
        return await self.get_object_state_flags(method="get_nsset_state_flags")

    async def check_dns_host(self, fqdn: str) -> bool:
        """Return whether DNS host exists.

        This method returns `True` if there is at least one registered nsset containing nameserver with `fqdn`.

        Args:
            fqdn: DNS host's fqdn.
        """
        request = CheckDnsHostRequest(fqdn=_idna_encode(fqdn))
        reply = await self.call("check_dns_host", request)
        return cast(bool, reply)

==> fred_regal-2.1.0.tar.gz: fred_regal-2.1.0/regal/nsset.py <==
"""Nsset client for registry service."""

import warnings
from collections.abc import AsyncIterable, Sequence
from datetime import datetime
from ipaddress import IPv4Address, IPv6Address, ip_address
from typing import Any, Optional, Union, cast

import grpc
from fred_api.registry.nsset.list_nssets_types_pb2 import ListNssetsRequest
from fred_api.registry.nsset.nsset_history_types_pb2 import NssetHistoryReply, NssetHistoryRequest
from fred_api.registry.nsset.nsset_info_types_pb2 import (
    CheckDnsHostRequest,
    DnsHost as DnsHostMessage,
    NssetIdReply,
    NssetIdRequest,
    NssetInfoReply,
    NssetInfoRequest,
)
from fred_api.registry.nsset.nsset_state_history_types_pb2 import NssetStateHistoryReply, NssetStateHistoryRequest
from fred_api.registry.nsset.nsset_state_types_pb2 import NssetStateReply, NssetStateRequest
from fred_api.registry.nsset.service_nsset_grpc_pb2_grpc import NssetStub
from fred_types import BaseModel, ContactRef, NssetHistoryId, NssetId, NssetRef

from .common import ObjectEvents, RegistryDecoder
from .exceptions import InvalidHistoryInterval, NssetDoesNotExist
from .object import ObjectStateFlag, RegistryObjectClient
from .utils import _idna_encode


class DnsHost(BaseModel):
    """Dns host info."""

    fqdn: str
    ip_addresses: list[Union[IPv4Address, IPv6Address]]


class Nsset(BaseModel):
    """Nsset info."""

    nsset_id: NssetId
    nsset_handle: str
    nsset_history_id: Optional[NssetHistoryId] = None
    sponsoring_registrar: Optional[str] = None
    dns_hosts: Sequence[DnsHost] = []
    technical_contacts: Sequence[ContactRef] = []
    technical_check_level: Optional[int] = None
    auth_info: Optional[str] = None
    events: Optional[ObjectEvents] = None


class NssetDecoder(RegistryDecoder):
    """Nsset service decoder."""

    def __init__(self) -> None:
        super().__init__()
        self.set_decoder(DnsHostMessage, self._decode_dns_host)
        self.set_exception_decoder(NssetIdReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetInfoReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetHistoryReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetHistoryReply.Exception.InvalidHistoryInterval, InvalidHistoryInterval)
        self.set_exception_decoder(NssetStateReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetStateHistoryReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetStateHistoryReply.Exception.InvalidHistoryInterval, InvalidHistoryInterval)

    def _decode_dns_host(self, value: DnsHostMessage) -> DnsHost:
        """Decode DNS host."""
        return DnsHost(
            fqdn=self.decode(value.fqdn),
            ip_addresses=[ip_address(address) for address in self.decode(value.ip_addresses)],
        )


class NssetClient(RegistryObjectClient[NssetId, NssetHistoryId]):
    """Nsset service gRPC client."""

    stub_cls = NssetStub
    decoder_cls = NssetDecoder
    service = "Nsset"
    object_id = "nsset_id"
    object_history_id = "nsset_history_id"
    object_handle = "nsset_handle"

    def __init__(self, netloc: str, credentials: Optional[grpc.ChannelCredentials] = None, **kwargs: Any) -> None:
        """Initialize nsset client instance.

        Args:
            netloc: Netloc of nsset service server.
            credentials: Optional credentials.
            kwargs: Other client arguments.
        """
        super().__init__(netloc, credentials=credentials, **kwargs)

    async def list(self, *, exclude_not_linked: bool = False) -> AsyncIterable[NssetRef]:
        """Iterate through all nssets in domain registry.

        Args:
            exclude_not_linked: Whether to exclude nssets without domains.

        Returns:
            Async iterable over domain refs.
        """
        request = ListNssetsRequest()
        request.exclude_nssets_without_domain = exclude_not_linked
        # XXX: Bug in python 3.9 coverage
        async for refs in self.call_stream("list_nssets", request):  # pragma: no cover
            for ref in refs:
                yield ref

    async def get_nsset_id(self, handle: str) -> NssetId:
        """Get nsset id.

        Args:
            handle: Nsset handle.

        Raises:
            NssetDoesNotExist: If nsset does not exist.
        """
        return NssetId(await self.get_object_id(request=NssetIdRequest(), method="get_nsset_id", handle=handle))

    async def get_nsset_info(self, nsset_id: NssetId, history_id: Optional[NssetHistoryId] = None) -> Nsset:
        """Get nsset info.

        Args:
            nsset_id: Nsset ID.
            history_id: Nsset history ID.

        Raises:
            NssetDoesNotExist: If nsset does not exist.
        """
        result = await self.get_object_info(
            request=NssetInfoRequest(),
            method="get_nsset_info",
            object_id=nsset_id,
            history_id=history_id,
        )
        result.pop("technical_contacts_old", None)
        return Nsset(**result)

    async def get_nsset_state(self, nsset_id: NssetId, *, internal: Optional[bool] = None) -> dict[str, bool]:
        """Get nsset state consisting of state flags and their boolean values.

        Args:
            nsset_id: Nsset ID.
            internal: Whether to include internal states in the result.

        Raises:
            NssetDoesNotExist: If nsset does not exist.
        """
        if internal is None:  # pragma: no cover
            warnings.warn(
                "Default value of internal flag will change to False in regal 2.0.", DeprecationWarning, stacklevel=2
            )
            internal = True

        return await self.get_object_state(
            request=NssetStateRequest(), method="get_nsset_state", object_id=nsset_id, internal=internal
        )

    async def get_nsset_history(
        self,
        nsset_id: NssetId,
        start: Optional[Union[NssetHistoryId, datetime]] = None,
        end: Optional[Union[NssetHistoryId, datetime]] = None,
    ) -> dict[str, Any]:
        """Get nsset history timeline.

        Args:
            nsset_id: Nsset ID.
            start: Datetime or history_id of history interval start.
            end: Datetime or history_id of history interval end.

        Raises:
            NssetDoesNotExist: If nsset does not exist.
            InvalidHistoryInterval: In case of invalid history interval.
        """
        return await self.get_object_history(
            request=NssetHistoryRequest(),
            method="get_nsset_history",
            object_id=nsset_id,
            start=start,
            end=end,
        )

    async def get_nsset_state_history(
        self,
        nsset_id: NssetId,
        start: Optional[Union[NssetHistoryId, datetime]] = None,
        end: Optional[Union[NssetHistoryId, datetime]] = None,
    ) -> dict[str, Any]:
        """Get nsset state history timeline.

        Args:
            nsset_id: Nsset ID.
            start: Datetime or history_id of history interval start.
            end: Datetime or history_id of history interval end.

        Raises:
            NssetDoesNotExist: If nsset does not exist.
            InvalidHistoryInterval: In case of invalid history interval.
        """
        return await self.get_object_state_history(
            request=NssetStateHistoryRequest(),
            method="get_nsset_state_history",
            object_id=nsset_id,
            start=start,
            end=end,
        )

    async def get_state_flags(self) -> dict[str, ObjectStateFlag]:
        """Get object state flags and their properties."""
        return cast(dict[str, ObjectStateFlag], await self.get_object_state_flags(method="get_nsset_state_flags"))  # type: ignore[call-overload]

    async def check_dns_host(self, fqdn: str) -> bool:
        """Return whether DNS host exists.

        This method returns `True` if there is at least one registered nsset containing nameserver with `fqdn`.

        Args:
            fqdn: DNS host's fqdn.
        """
        request = CheckDnsHostRequest(fqdn=_idna_encode(fqdn))
        reply = await self.call("check_dns_host", request)
        return cast(bool, reply)
